"""
JSON-RPC (especially v2) prescribes specific error messaging and codes
Here we store the error codes and code that helps us format the errors.

This file is part of `jsonrpcparts` project. See project's source for license and copyright.
"""

#JSON-RPC 2.0 error-codes
PARSE_ERROR = -32700
INVALID_REQUEST = -32600
METHOD_NOT_FOUND = -32601
INVALID_METHOD_PARAMS = -32602  #invalid number/type of parameters
INTERNAL_ERROR = -32603  #"all other errors"

#additional error-codes
PROCEDURE_EXCEPTION = -32000
AUTHENTIFICATION_ERROR = -32001
PERMISSION_DENIED = -32002
INVALID_PARAM_VALUES = -32003

#human-readable messages
ERROR_MESSAGE = {
    PARSE_ERROR:"Parse error.",
    INVALID_REQUEST:"Invalid Request.",
    METHOD_NOT_FOUND:"Method not found.",
    INVALID_METHOD_PARAMS:"Invalid parameters.",
    INTERNAL_ERROR:"Internal error.",

    PROCEDURE_EXCEPTION:"Procedure exception.",
    AUTHENTIFICATION_ERROR:"Authentification error.",
    PERMISSION_DENIED:"Permission denied.",
    INVALID_PARAM_VALUES:"Invalid parameter values."
}


class RPCError(Exception):
    """Base class for rpc-errors."""


class RPCFault(RPCError):
    """RPC error/fault package received.

    This exception can also be used as a class, to generate a
    RPC-error/fault message.

    :Variables:
        - error_code:   the RPC error-code
        - error_string: description of the error
        - error_data:   optional additional information
                        (must be json-serializable)
    :TODO: improve __str__
    """

    error_code = None

    def __init__(self, error_data=None, request_id=None, *args, **kw):
        RPCError.__init__(self, ERROR_MESSAGE.get(self.error_code, None), *args, **kw)

        self.error_data = error_data
        self.request_id = request_id

    def __str__(self):
        return repr(self)

    def __repr__(self):
        return( "<RPCFault %s: %s (%s)>" % (self.error_code, repr(self.message), repr(self.error_data)) )


class RPCParseError(RPCFault):
    error_code = PARSE_ERROR


class RPCInvalidRequest(RPCFault):
    error_code = INVALID_REQUEST


class RPCMethodNotFound(RPCFault):
    """Method not found. (METHOD_NOT_FOUND)"""
    error_code = METHOD_NOT_FOUND


class RPCInvalidMethodParams(RPCFault):
    error_code = INVALID_METHOD_PARAMS


class RPCInternalError(RPCFault):
    """Internal error. (INTERNAL_ERROR)"""
    error_code = INTERNAL_ERROR


class RPCProcedureException(RPCFault):
    """Procedure exception. (PROCEDURE_EXCEPTION)"""
    error_code = PROCEDURE_EXCEPTION


class RPCAuthentificationError(RPCFault):
    """AUTHENTIFICATION_ERROR"""
    error_code = AUTHENTIFICATION_ERROR


class RPCPermissionDenied(RPCFault):
    """PERMISSION_DENIED"""
    error_code = PERMISSION_DENIED


class RPCInvalidParamValues(RPCFault):
    """INVALID_PARAM_VALUES"""
    error_code = INVALID_PARAM_VALUES


ERROR_CODE_CLASS_MAP = {
    klass.error_code:klass \
    for klass in [
        RPCParseError,RPCInvalidRequest,RPCMethodNotFound,
        RPCInvalidMethodParams,RPCInternalError,RPCProcedureException,
        RPCAuthentificationError,RPCPermissionDenied,RPCInvalidParamValues
    ]
}

assert len(ERROR_CODE_CLASS_MAP.keys()) == len(ERROR_MESSAGE.keys())



"""
This module contains code that simplifies the job of JSON-RPC clients.

This file is part of `jsonrpcparts` project. See project's source for license and copyright.
"""
import json
import requests

from . import errors
from . import JSONRPC20Serializer

class Client(object):

    def __init__(self, data_serializer=JSONRPC20Serializer):
        """
        :Parameters:
            - data_serializer: a data_structure+serializer-instance
        """
        self._in_batch_mode = False
        self._requests = []
        self._data_serializer = data_serializer

    # Context manager API
    def __enter__(self):
        self._in_batch_mode = True
        self._requests = []
        return self

    # Context manager API
    def __exit__(self, *args):
        self._in_batch_mode = False
        self._requests = []
        pass

    def call(self, method, *args, **kw):
        """
        In context of a batch we return the request's ID
        else we return the actual json
        """
        if args and kw:
            raise ValueError("JSON-RPC method calls allow only either named or positional arguments.")
        if not method:
            raise ValueError("JSON-RPC method call requires a method name.")

        request = self._data_serializer.assemble_request(
            method, args or kw or None
        )

        if self._in_batch_mode:
            self._requests.append(request)
            return request.get('id')
        else:
            return request

    def notify(self, method, *args, **kw):
        if args and kw:
            raise ValueError("JSON-RPC method calls allow only either named or positional arguments.")
        if not method:
            raise ValueError("JSON-RPC method call requires a method name.")

        request = self._data_serializer.assemble_request(
            method, args or kw or None, notification=True
        )

        if self._in_batch_mode:
            self._requests.append(request)
        else:
            return request

    def get_batched(self):
        if not self._in_batch_mode:
            return []
        else:
            return self._requests

class WebClient(Client):
    """
    This class internalizes the JSON RPC Client class which allows batching of requests
    and adds code that turns RPC call / notification run into HTTP requests.
    """

    def __init__(self, rpc_server_url, data_serializer=JSONRPC20Serializer):
        """
        :Parameters:
            - prc_server_url: string
            - data_serializer: a data_structure+serializer-instance
        """
        super(WebClient, self).__init__(data_serializer)
        self._rpc_server_url = rpc_server_url

    def _communicate(self, request_json, expect_response):
        response = requests.post(
            self._rpc_server_url,
            data=json.dumps(request_json),
            headers={'Content-Type': 'application/json'}
        )

        assert response.status_code == 200
        if expect_response:
            return response.json()

    def notify(self, method, *args, **kw):
        """

        """
        self._communicate(
            super(WebClient, self).notify(method, *args, **kw),
            expect_response=False
        )

    def call(self, method, *args, **kw):
        """
`
        """
        json_rpc_response = self._communicate(
            super(WebClient, self).call(method, *args, **kw),
            expect_response=True
        )

        #base['error'] = {
        #    'message':error.message,
        #    'code':error.error_code,
        #    'data':'error_data'
        #}

        if 'error' in json_rpc_response:
            error_class = errors.ERROR_CODE_CLASS_MAP.get(
                json_rpc_response['error'].get('code')
            ) or errors.RPCError
            if issubclass(error_class, errors.RPCFault):
                raise error_class(
                    json_rpc_response['error'].get('data'),
                    json_rpc_response.get('id'),
                    json_rpc_response['error'].get('message')
                )
            else:
                raise error_class(
                    json_rpc_response['error'].get('message')
                )

        return json_rpc_response['result']



"""
JSON-RPC (especially v2) prescribes that the message format adheres
to a specific format/schema. This module contains code that helps
in serializing data (including errors) into JSON-RPC message format.

This file is part of `jsonrpcparts` project. See project's source for license and copyright.
"""

import json
import time
import random

from . import errors

def clean_dict_keys(d):
    """Convert all keys of the dict 'd' to (ascii-)strings.

    :Raises: UnicodeEncodeError
    """
    new_d = {}
    for (k, v) in d.iteritems():
        new_d[str(k)] = v
    return new_d

#----------------------
# JSON-RPC 1.0

class JSONRPC10Serializer:
    """JSON-RPC V1.0 data-structure / serializer

    This implementation is quite liberal in what it accepts: It treats
    missing "params" and "id" in Requests and missing "result"/"error" in
    Responses as empty/null.

    :SeeAlso:   JSON-RPC 1.0 specification
    :TODO:      catch json.dumps not-serializable-exceptions
    """

    @staticmethod
    def assemble_request(method, params=tuple(), id=0):
        """serialize JSON-RPC-Request

        :Parameters:
            - method: the method-name (str/unicode)
            - params: the parameters (list/tuple)
            - id:     if id=None, this results in a Notification
        :Returns:   | {"method": "...", "params": ..., "id": ...}
                    | "method", "params" and "id" are always in this order.
        :Raises:    TypeError if method/params is of wrong type or
                    not JSON-serializable
        """
        if not isinstance(method, (str, unicode)):
            raise TypeError('"method" must be a string (or unicode string).')
        if not isinstance(params, (tuple, list)):
            raise TypeError("params must be a tuple/list.")

        return {
            "method": method,
            "params": params,
            "id": id
        }

    @staticmethod
    def assemble_notification_request(method, params=tuple()):
        """serialize a JSON-RPC-Notification

        :Parameters: see dumps_request
        :Returns:   | {"method": "...", "params": ..., "id": null}
                    | "method", "params" and "id" are always in this order.
        :Raises:    see dumps_request
        """
        if not isinstance(method, (str, unicode)):
            raise TypeError('"method" must be a string (or unicode string).')
        if not isinstance(params, (tuple, list)):
            raise TypeError("params must be a tuple/list.")

        return {
            "method": method,
            "params": params,
            "id": None
        }

    @staticmethod
    def assemble_response(result, id=None):
        """serialize a JSON-RPC-Response (without error)

        :Returns:   | {"result": ..., "error": null, "id": ...}
                    | "result", "error" and "id" are always in this order.
        :Raises:    TypeError if not JSON-serializable
        """
        return {
            "result": result,
            "error": None,
            "id": id
        }

    @staticmethod
    def assemble_error_response(error, id=None):
        """serialize a JSON-RPC-Response-error

        Since JSON-RPC 1.0 does not define an error-object, this uses the
        JSON-RPC 2.0 error-object.

        :Parameters:
            - error: a RPCFault instance
        :Returns:   | {"result": null, "error": {"code": error_code, "message": error_message, "data": error_data}, "id": ...}
                    | "result", "error" and "id" are always in this order, data is omitted if None.
        :Raises:    ValueError if error is not a RPCFault instance,
                    TypeError if not JSON-serializable
        """
        if not isinstance(error, errors.RPCFault):
            raise ValueError("""error must be a RPCFault-instance.""")
        if error.error_data is None:
            return {
                "result": None,
                "error": {
                    "code": error.error_code,
                    "message": error.message
                },
                "id": id
            }
        else:
            return {
                "result": None,
                "error": {
                    "code":error.error_code,
                    "message": error.message,
                    "data": error.error_data
                },
                "id": id
            }

    @staticmethod
    def parse_request(jsonrpc_message):
        """We take apart JSON-RPC-formatted message as a string and decompose it
        into a dictionary object, emitting errors if parsing detects issues with
        the format of the message.

        :Returns:   | [method_name, params, id] or [method_name, params]
                    | params is a tuple/list
                    | if id is missing, this is a Notification
        :Raises:    RPCParseError, RPCInvalidRPC, RPCInvalidMethodParams
        """
        try:
            data = json.loads(jsonrpc_message)
        except ValueError, err:
            raise errors.RPCParseError("No valid JSON. (%s)" % str(err))

        if not isinstance(data, dict):
            raise errors.RPCInvalidRPC("No valid RPC-package.")
        if "method" not in data:
            raise errors.RPCInvalidRPC("""Invalid Request, "method" is missing.""")
        if not isinstance(data["method"], (str, unicode)):
            raise errors.RPCInvalidRPC("""Invalid Request, "method" must be a string.""")
        if "id" not in data:
            data["id"] = None #be liberal
        if "params" not in data:
            data["params"] = ()     #be liberal
        if not isinstance(data["params"], (list, tuple)):
            raise errors.RPCInvalidRPC("""Invalid Request, "params" must be an array.""")
        if len(data) != 3:
            raise errors.RPCInvalidRPC("""Invalid Request, additional fields found.""")

        # notification / request
        if data["id"] is None:
            return data["method"], data["params"] #notification
        else:
            return data["method"], data["params"], data["id"] #request

    @staticmethod
    def parse_response(jsonrpc_message):
        """de-serialize a JSON-RPC Response/error

        :Returns: | [result, id] for Responses
        :Raises:  | RPCFault+derivates for error-packages/faults, RPCParseError, RPCInvalidRPC
                  | Note that for error-packages which do not match the
                    V2.0-definition, RPCFault(-1, "Error", RECEIVED_ERROR_OBJ)
                    is raised.
        """
        try:
            data = json.loads(jsonrpc_message)
        except ValueError, err:
            raise errors.RPCParseError("No valid JSON. (%s)" % str(err))
        if not isinstance(data, dict):
            raise errors.RPCInvalidRPC("No valid RPC-package.")
        if "id" not in data:
            raise errors.RPCInvalidRPC("""Invalid Response, "id" missing.""")
        if "result" not in data:
            data["result"] = None #be liberal
        if "error" not in data:
            data["error"] = None #be liberal
        if len(data) != 3:
            raise errors.RPCInvalidRPC("""Invalid Response, additional or missing fields.""")

        #error
        if data["error"] is not None:
            if data["result"] is not None:
                raise errors.RPCInvalidRPC("""Invalid Response, one of "result" or "error" must be null.""")
            #v2.0 error-format
            if (
                isinstance(data["error"], dict) and
                "code" in data["error"] and
                "message" in data["error"] and
                (
                    len(data["error"]) == 2 or
                    ("data" in data["error"] and len(data["error"])==3)
                )
            ):
                if "data" not in data["error"]:
                    error_data = None
                else:
                    error_data = data["error"]["data"]

                if data["error"]["code"] == errors.PARSE_ERROR:
                    raise errors.RPCParseError(error_data)
                elif data["error"]["code"] == errors.INVALID_REQUEST:
                    raise errors.RPCInvalidRPC(error_data)
                elif data["error"]["code"] == errors.METHOD_NOT_FOUND:
                    raise errors.RPCMethodNotFound(error_data)
                elif data["error"]["code"] == errors.INVALID_METHOD_PARAMS:
                    raise errors.RPCInvalidMethodParams(error_data)
                elif data["error"]["code"] == errors.INTERNAL_ERROR:
                    raise errors.RPCInternalError(error_data)
                elif data["error"]["code"] == errors.PROCEDURE_EXCEPTION:
                    raise errors.RPCProcedureException(error_data)
                elif data["error"]["code"] == errors.AUTHENTIFICATION_ERROR:
                    raise errors.RPCAuthentificationError(error_data)
                elif data["error"]["code"] == errors.PERMISSION_DENIED:
                    raise errors.RPCPermissionDenied(error_data)
                elif data["error"]["code"] == errors.INVALID_PARAM_VALUES:
                    raise errors.RPCInvalidParamValues(error_data)
                else:
                    raise errors.RPCFault(data["error"]["code"], data["error"]["message"], error_data)
            #other error-format
            else:
                raise errors.RPCFault(-1, "Error", data["error"])
        #result
        else:
            return data["result"], data["id"]

#----------------------
# JSON-RPC 2.0

class JSONRPC20Serializer:

    @staticmethod
    def assemble_request(method, params=None, notification=False):
        """serialize JSON-RPC-Request

        :Parameters:
            - method: the method-name (str/unicode)
            - params: the parameters (None/list/tuple/dict)
            - notification: bool
        :Returns:   | {"jsonrpc": "2.0", "method": "...", "params": ..., "id": ...}
                    | "jsonrpc", "method", "params" and "id" are always in this order.
                    | "params" is omitted if empty
        :Raises:    TypeError if method/params is of wrong type or
                    not JSON-serializable
        """

        if not isinstance(method, (str, unicode)):
            raise TypeError('"method" must be a string (or unicode string).')
        if params and not isinstance(params, (tuple, list, dict)):
            raise TypeError("params must be a tuple/list/dict or None.")

        base = {
            "jsonrpc": "2.0",
            "method": method
        }

        if params:
            base["params"] = params

        if not notification:
            base['id'] = long(time.time() * 1000)

        return base

    @staticmethod
    def assemble_response(result, request_id):
        return {
            "jsonrpc": "2.0", 
            "result": result, 
            "id": request_id
        }

    @staticmethod
    def assemble_error_response(error):

        if not isinstance(error, errors.RPCFault):
            raise ValueError("""error must be a RPCFault-instance.""")

        if error.error_data is None:
            return {
                "jsonrpc": "2.0", 
                "error": {
                    "code":error.error_code, 
                    "message":error.message
                }, 
                "id": error.request_id
            }
        else:
            return {
                "jsonrpc": "2.0", 
                "error": {
                    "code":error.error_code, 
                    "message": error.message,
                    "data": error.error_data
                }, 
                "id": error.request_id
            }

    @classmethod
    def _parse_single_request(cls, request_data):
        """

        :Returns:   | [method_name, params, id]
                    | method (str)
                    | params (tuple/list or dict)
                    | id (str/int/None) (None means this is Notification)
        :Raises:    RPCParseError, RPCInvalidRPC, RPCInvalidMethodParams
        """

        request_id = request_data.get('id', None) # Notifications don't have IDs

        for argument in ['jsonrpc', 'method']:
            if argument not in request_data:
                raise errors.RPCInvalidRequest('argument "%s" missing.' % argument, request_id)
            if not isinstance(request_data[argument], (str, unicode)):
                raise errors.RPCInvalidRequest('value of argument "%s" must be a string.' % argument, request_id)

        if request_data["jsonrpc"] != "2.0":
            raise errors.RPCInvalidRequest('Invalid jsonrpc version.', request_id)

        if "params" in request_data:
            if not isinstance(request_data["params"], (list, tuple, dict)):
                raise errors.RPCInvalidMethodParams(
                    'value of argument "parameter" is of non-supported type %s' % type(request_data["params"]),
                    request_id
                )

        return (
            request_data["method"],
            request_data.get("params", None),
            request_id
        )

    @classmethod
    def _parse_single_request_trap_errors(cls, request_data):
        """Traps exceptions generated by __parse_single_request and
        converts them into values of request_id and error in the
        returned tuple.

        :Returns: (method_name, params_object, request_id, error)
                Where:
                - method_name is a str (or None when error is set)
                - params_object is one of list/tuple/dict/None
                - request_id is long/int/string/None
                - error is an instance of errors.RPCFault subclass or None
        """
        try:
            method, params, request_id = cls._parse_single_request(request_data)
            return method, params, request_id, None
        except errors.RPCFault as ex:
            return None, None, ex.request_id, ex

    @classmethod
    def parse_request(cls, request_string):
        """JSONRPC allows for **batch** requests to be communicated
        as array of dicts. This method parses out each individual
        element in the batch and returns a list of tuples, each
        tuple a result of parsing of each item in the batch.

        :Returns:   | tuple of (results, is_batch_mode_flag)
                    | where:
                    | - results is a tuple describing the request
                    | - Is_batch_mode_flag is a Bool indicating if the
                    |   request came in in batch mode (as array of requests) or not.

        :Raises:    RPCParseError, RPCInvalidRequest
        """
        try:
            batch = json.loads(request_string)
        except ValueError as err:
            raise errors.RPCParseError("No valid JSON. (%s)" % str(err))

        if isinstance(batch, (list, tuple)) and batch:
            # batch is true batch.
            # list of parsed request objects, is_batch_mode_flag
            return [cls._parse_single_request_trap_errors(request) for request in batch], True
        elif isinstance(batch, dict):
            # `batch` is actually single request object
            return [cls._parse_single_request_trap_errors(batch)], False

        raise errors.RPCInvalidRequest("Neither a batch array nor a single request object found in the request.")

    @classmethod
    def _parse_single_response(cls, response_data):
        """de-serialize a JSON-RPC Response/error

        :Returns: | [result, id] for Responses
        :Raises:  | RPCFault+derivates for error-packages/faults, RPCParseError, RPCInvalidRPC
        """

        if not isinstance(response_data, dict):
            raise errors.RPCInvalidRequest("No valid RPC-package.")

        if "id" not in response_data:
            raise errors.RPCInvalidRequest("""Invalid Response, "id" missing.""")

        request_id = response_data['id']

        if "jsonrpc" not in response_data:
            raise errors.RPCInvalidRequest("""Invalid Response, "jsonrpc" missing.""", request_id)
        if not isinstance(response_data["jsonrpc"], (str, unicode)):
            raise errors.RPCInvalidRequest("""Invalid Response, "jsonrpc" must be a string.""")
        if response_data["jsonrpc"] != "2.0":
            raise errors.RPCInvalidRequest("""Invalid jsonrpc version.""", request_id)

        error = response_data.get('error', None)
        result = response_data.get('result', None)

        if error and result:
            raise errors.RPCInvalidRequest("""Invalid Response, only "result" OR "error" allowed.""", request_id)

        if error:
            if not isinstance(error, dict):
                raise errors.RPCInvalidRequest("Invalid Response, invalid error-object.", request_id)

            if not ("code" in error and "message" in error):
                raise errors.RPCInvalidRequest("Invalid Response, invalid error-object.", request_id)

            error_data = error.get("data", None)

            if error['code'] in errors.ERROR_CODE_CLASS_MAP:
                raise errors.ERROR_CODE_CLASS_MAP[error['code']](error_data, request_id)
            else:
                error_object = errors.RPCFault(error_data, request_id)
                error_object.error_code = error['code']
                error_object.message = error['message']
                raise error_object

        return result, request_id

    @classmethod
    def _parse_single_response_trap_errors(cls, response_data):
        try:
            result, request_id = cls._parse_single_response(response_data)
            return result, request_id, None
        except errors.RPCFault as ex:
            return None, ex.request_id, ex

    @classmethod
    def parse_response(cls, response_string):
        """JSONRPC allows for **batch** responses to be communicated
        as arrays of dicts. This method parses out each individual
        element in the batch and returns a list of tuples, each
        tuple a result of parsing of each item in the batch.

        :Returns:   | tuple of (results, is_batch_mode_flag)
                    | where:
                    | - results is a tuple describing the request
                    | - Is_batch_mode_flag is a Bool indicating if the
                    |   request came in in batch mode (as array of requests) or not.

        :Raises:    RPCParseError, RPCInvalidRequest
        """
        try:
            batch = json.loads(response_string)
        except ValueError as err:
            raise errors.RPCParseError("No valid JSON. (%s)" % str(err))

        if isinstance(batch, (list, tuple)) and batch:
            # batch is true batch.
            # list of parsed request objects, is_batch_mode_flag
            return [cls._parse_single_response_trap_errors(response) for response in batch], True
        elif isinstance(batch, dict):
            # `batch` is actually single response object
            return [cls._parse_single_response_trap_errors(batch)], False

        raise errors.RPCParseError("Neither a batch array nor a single response object found in the response.")



"""
In JSON-RPC it makes sense to group all server-side methods for a given
rpc channel into a "collection" that is inspected for supported method
when it comes time to handle a request.

This module contains code that allows one to have this generic collection
that supports discovery and use of the registered methods.

This file is part of `jsonrpcparts` project. See project's source for license and copyright.
"""
import json

from . import errors
from .serializers import JSONRPC20Serializer

class JSONPRCCollection(dict):
    """
    A dictionary-like collection that helps with registration
    and use (calling of) JSON-RPC methods.
    """

    def register_class(self, instance, name=None):
        """Add all functions of a class-instance to the RPC-services.

        All entries of the instance which do not begin with '_' are added.

        :Parameters:
            - myinst: class-instance containing the functions
            - name:   | hierarchical prefix.
                      | If omitted, the functions are added directly.
                      | If given, the functions are added as "name.function".
        :TODO:
            - only add functions and omit attributes?
            - improve hierarchy?
        """
        prefix_name = name or instance.__class__.__name__

        for e in dir(instance):
            if e[0][0] != "_":
                self.register_function(
                    getattr(instance, e),
                    name="%s.%s" % (prefix_name, e)
                )

    def register_function(self, function, name=None):
        """Add a function to the RPC-services.

        :Parameters:
            - function: function to add
            - name:     RPC-name for the function. If omitted/None, the original
                        name of the function is used.
        """
        if name:
            self[name] = function
        else:
            self[function.__name__] = function


class JSONPRCApplication(JSONPRCCollection):

    def __init__(self, data_serializer=JSONRPC20Serializer, *args, **kw):
        """
        :Parameters:
            - data_serializer: a data_structure+serializer-instance
        """
        super(JSONPRCApplication, self).__init__(*args, **kw)
        self._data_serializer = data_serializer

    def process_requests(self, requests):
        """
        Turns a list of request objects into a list of
        response objects.
        """

        ds = self._data_serializer

        responses = []
        for method, params, request_id, error in requests:

            if error: # these are request message validation errors
                if error.request_id: # no ID = Notification. We don't reply
                    responses.append(ds.assemble_error_response(error))
                continue

            if method not in self:
                if request_id:
                    responses.append(ds.assemble_error_response(
                        errors.RPCMethodNotFound(
                            'Method "%s" is not found.' % method,
                            request_id
                        )
                    ))
                continue

            try:
                if isinstance(params, dict):
                    result = self[method](**params)
                else:
                    result = self[method](*params)
                if request_id:
                    responses.append(ds.assemble_response(result, request_id))
            except errors.RPCFault as ex:
                if request_id:
                    responses.append(ds.assemble_error_response(ex))
            except Exception as ex:
                if request_id:
                    responses.append(ds.assemble_error_response(
                        errors.RPCInternalError(
                            'While processing the follwoing message ("%s","%s","%s") ' % (method, params, request_id) +\
                            'encountered the following error message "%s"' % ex.message
                        )
                    ))

        return responses

    def handle_request_string(self, request_string):
        """Handle a RPC-Request.

        :Parameters:
            - request_string: the received rpc-string
        :Returns: the encoded (serialized as string) JSON of the response
        """

        ds = self._data_serializer

        try:
            requests, is_batch_mode = ds.parse_request(request_string)
        except errors.RPCFault as ex:
            return json.dumps(ds.assemble_error_response(ex))
        except Exception as ex:
            return json.dumps(ds.assemble_error_response(
                errors.RPCInternalError(
                    'While processing the follwoing message "%s" ' % request_string +\
                    'encountered the following error message "%s"' % ex.message
                )
            ))

        responses = self.process_requests(requests)

        if not responses:
            return None

        try:
            if is_batch_mode:
                return json.dumps(responses)
            else:
                return json.dumps(responses[0])
        except Exception as ex:
            return json.dumps(
                ds.assemble_error_response(
                    errors.RPCInternalError(
                        'While processing the follwoing message "%s" ' % request_string +\
                        'encountered the following error message "%s"' % ex.message
                    )
                )
            )



import sys
from setuptools import setup, find_packages

#from pip.req import parse_requirements
#install_reqs = parse_requirements('requirements.txt')
#requirements = [str(ir.req) for ir in install_reqs]

with open('jsonrpcparts/VERSION') as version_file:
    version = version_file.read().strip()

long_description = """
JSON-RPC Parts is a library of composable components one would need to assemble a JSON-RPC server or client.

The parts provided are JSON-RPC message parser and serializer, a generic request handler collection, a WSGI-specific request handler and bits and pieces.

This JSON-RPC Parts collection supports both, JSON-RPC v.1.0 and v.2.0 including "batch" mode for v.2.0.

The parts are split into separate modules that can be used separately from this collection.

Since this collection is MIT-licensed, you are free grab a part of this code and use it in alsmost any.
"""

project_home = 'http://github.com/dvdotsenko/jsonrpc.py'


if __name__ == "__main__":
    setup(
        name='jsonrpcparts',
        description='JSON-RPC client and server components',
        long_description=long_description,
        version=version,
        author='Daniel Dotsenko',
        author_email='dotsa@hotmail.com',
        url=project_home,
        download_url=project_home+'/tarball/master',
        classifiers=[
            "Development Status :: 4 - Beta",
            "Environment :: Web Environment",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 2.6",
            "Programming Language :: Python :: 2.7",
            "Topic :: Internet",
            "Topic :: Internet :: WWW/HTTP",
            "Topic :: Internet :: WWW/HTTP :: WSGI",
            "Topic :: Internet :: WWW/HTTP :: WSGI :: Application"
        ],
        keywords = ['JSON', 'jsonrpc', 'rpc', 'wsgi'],
        license='MIT',
        packages=find_packages(),
        package_data={
            'jsonrpcparts':['VERSION'],
        },
        include_package_data=True,
        install_requires=['requests']
    )

    # Next:
    # python setup.py register
    # python setup.py sdist upload



"""
This module contains code that allows one to have this generic collection
that supports discovery and use of the registered methods.

This file is part of `jsonrpcparts` project. See project's source for license and copyright.
"""
from . import JSONPRCApplication

class JSONPRCWSGIApplication(JSONPRCApplication):

    def handle_wsgi_request(self, environ, start_response):

        assert 'CONTENT_TYPE' in environ
        assert environ['CONTENT_TYPE'] == 'application/json'

        content_length = None
        if 'CONTENT_LENGTH' in environ:
            content_length = environ['CONTENT_LENGTH']
            if content_length:
                content_length = long(content_length)

        input_stream = environ['wsgi.input']

        def get_next_chunk(content_length):
            """Per old WSGI spec, PEP 333, if content length is provided, clients
            are requested to read only that much of input. Reading more than that
            would forever block on old servers.

            Per new WSGI spec, PEP 3333, the input pipe will issue EOF at appropriate time.
            So, there caring about content length is no longer needed for the client,
            but it's still nice to do if content length value is provided.

            Here we reduce the number of the content length remaining by the size of the
            chunk sucked out of the pipe.
            """
            if content_length is None:
                return input_stream.read(), content_length
            else:
                chunk = input_stream.read(content_length)
                return chunk, content_length - len(chunk)

        chunks = []
        chunk, content_length = get_next_chunk(content_length)
        while chunk:
            chunks.append(chunk)
            chunk, content_length = get_next_chunk(content_length)

        request_string = ''.join(chunks)
        response_string = self.handle_request_string(request_string)

        if response_string:
            headers = [
                ('Content-Type', 'application/json'),
                ('Content-Length', len(response_string))
            ]
            start_response('200 OK', headers)
            return [response_string]
        else:
            headers = [
                ('Content-Type', 'text/plain'),
                ('Content-Length', 0)
            ]
            start_response('200 OK', headers)
            return []

    def __call__(self, environ, start_response):
        return self.handle_wsgi_request(environ, start_response)
